(defun clt-illustrator (y &rest args
  &key (sample-sizes '(2 3 4 5 6 7 8 16 32 64)) (reps 500))
"Functions args:  (y &key (sample-sizes '(2 3 4 8 16 32)) (reps 500))
Draws a histogram of y, and adds a normal density with the same mean and sd.
Opens a second histogram window with data equal to reps sample means of size
sample-sizes, with the corresponding normal density added. A slide bar allows
changing the sample size."
 (flet ((get-means (n) (mapcar #'(lambda (r) (mean
                  (select y (floor (* (length y) (uniform-rand n))))))
                  (iseq reps))))
  (let* ((h1 (apply #'histogram y :title "Raw data" args))
         (ybar (mean y))
         (ysd (standard-deviation y))
         (j 0)
         (n (select sample-sizes j))
         (h2 (histogram (get-means n) :title "Histogram of means"
               :variable-labels (list (format nil "Means of ~a" n)) 
               :show nil))
         (fixed-range t)
         (rangex (send h2 :range 0)))
   (send h1 :plot-controls)
   (send h1 :add-function
       #'(lambda (z) (/ (normal-dens (/ (- z ybar) ysd)) ysd))
       (- ybar (* 3.5 ysd)) (+ ybar (* 3.5 ysd)))
   (defmeth h2 :install-means-control ()
     (let ((c (first (send self :add-overlay
                 (send slider-control-proto :new sample-sizes
                    :title "Sample size" :popup t
                    :location (send self :locate-next-control :height 2)
                    :length (send self :slider-width))))))
      (defmeth c :menu (&optional (title "Scaling Option")) 
        (let* ((graph (send self :graph)))
          (when (null (slot-value 'menu)) 
            (let* ((menu (send menu-proto :new title)))
                  (apply #'send menu :append-items (send self :get-menu-items))
                  (send self :slot-value 'menu menu)))
              (slot-value 'menu)))
      (defmeth c :shift-click ()
        (call-method smoother-proto :shift-click))
      (defmeth c :get-menu-items ()
        (let* ((adjust (send menu-item-proto :new "Adjust Scale"
              :action #'(lambda () (setf fixed-range (null fixed-range))))))
          (defmeth adjust :update ()
             (send adjust :mark (null fixed-range)))
          (list adjust)))
      (defmeth c :do-action (a)
        (send h2 :start-buffering)
        (send h2 :clear-lines)
        (send h2 :point-coordinate 0 (iseq reps) (get-means a))
        (send h2 :normal-density)
        (send h2 :variable-label 0 (format nil "Means of ~a" a))
        (send h2 :adjust-to-data)
        (if fixed-range (apply #'send h2 :range 0 rangex))
        (send h2 :buffer-to-screen))))
   (defmeth h2 :plot-controls ()
     (call-method graph-proto :plot-controls)
     (send self :install-bin-control)
     (send self :install-kernel-control)
     (send self :install-means-control)
     t)
   (send h2 :plot-controls)
   (defmeth h2 :normal-density ()
     (let* ((y (send h2 :point-coordinate 0 (iseq reps)))
            (ybar (mean y))
            (ysd (standard-deviation y)))
      (send self :add-function
        #'(lambda (z) (/ (normal-dens (/ (- z ybar) ysd)) ysd))
        (- ybar (* 3.5 ysd)) (+ ybar (* 3.5 ysd)))))
   (send h2 :normal-density)
   (apply #'send h2 :location (+ (list 0 45)
     (send h1 :location) (list 0 (second (send h1 :size)))))
   (setf g1 h1)
   (setf g2 h2)
   (send h2 :show-window))))


(defun clt-menu ()
   (let* ((menu (send menu-proto :new "CLT"))
          (items '( "CEO salaries"
                    "N(10,4) random numbers"
                    "Highly skewed exponential data"
                    "Cauchy data"
                    "Quit"))
          (actions (list #'cltceo #'cltnorm104 #'cltexp #'cltcauchy
                   #'(lambda () (send menu :remove)))))
   (apply #'send menu :append-items
          (mapcar #'(lambda (i a) (send menu-item-proto :new i :action a))
            items actions))
   (send menu :install)
   menu))

(defun clthay ()
 (let ((vol (* .001
(QUOTE (2853 2702 3099 1306 2294 2725 3309 2790 2756 5237.92 3149.82 5498.46
3397.83 3007.56 4574.29 6228.59 2318.64 3176.71 2352.31 2174.44 2694.72
3333.53 4328.92 2115.04 2489.08 2296.65 3117.21 4088.36 4180.88 2318.19 1946.9
2479.89 3174.8 2151.54 3475.68 4393.08 2819.5 3703.49 2742.81 3002.4 1854.19
1982.07 2470.86 1203.15 2843.84 2636.25 1998.39 2005.03 2568.76 2161.18 2112.2
3009.33 1992.24 2746.98 2238.27 1747.47 2863.91 3593.47 2435.48 2430.18
2590.07 3577.68 3299.24 1986.14 3109.04 2821.56 2932.24 3304.63 2565.46
4509.93 4804.01 4241.8 4516.1 5011.62 2110.73 2775.7 3927.9 4212.77 3562.64
2853.96 3294.38 1689.54 2228.84 2362.61 3088.28 3820.79 3126.64 3624.75
3023.97 6045.42 3100.11 3378.07 3040.29 2252.16 3552.61 2635.9 3201.41 2590.21
3743.55 3858.03 3829.44 2556.44 3119.07 2122.38 2921.92 2936.35 2427.66
2069.38 1899.54 4289.28 2407.39 3097.99 3893.67 2238.66 2314.79 2667.07
2582.07 3426.5 2307.34 3960.41)))))
  (clt-illustrator vol :variable-labels '("Vol/1000"))))

(defun cltceo ()
 (let ((sal (* .001
(QUOTE (100 103 133 155 183 202 224 241 247 250 264
270 275 280 282 292 295 296 299 300 302 306 307 311 311 324 326 326 327 330
336 336 337 338 341 343 348 359 361 364 368 369 371 371 371 375 375 375 376
383 386 394 395 398 399 400 400 400 400 400 401 401 401 405 409 409 410 411
413 418 419 420 421 425 427 431 433 439 440 440 444 445 445 446 451 453 453
456 462 462 464 466 466 467 469 472 476 479 480 482 482 483 484 485 487 487
487 492 492 493 495 495 496 497 500 500 502 502 504 505 506 510 510 510 511
511 511 514 514 515 515 518 519 522 522 522 524 525 527 533 539 540 541 542
543 543 544 545 546 546 547 551 554 554 556 557 558 565 567 568 569 569 570
573 574 574 575 576 580 581 581 583 588 588 593 593 593 596 596 597 597 599
599 600 601 603 606 611 612 613 617 618 619 621 621 621 624 625 634 635 635
637 642 642 646 647 648 651 654 654 656 658 660 661 668 669 669 669 670 670
675 676 676 683 684 686 687 688 692 692 693 696 696 701 701 701 704 705 708
709 710 711 711 721 722 724 725 727 729 731 736 741 744 744 744 746 747 748
750 750 750 750 752 753 754 756 762 765 768 770 772 774 774 776 777 779 782
782 784 784 784 789 800 803 804 804 807 808 809 810 811 823 825 838 841 842
842 844 851 858 860 863 863 866 867 871 873 874 875 875 882 886 888 889 889
893 898 902 905 908 912 914 915 917 917 921 923 924 925 925 928 929 932 935
937 945 946 946 946 948 950 950 954 959 960 960 962 963 964 964 964 968 976
981 991 992 993 993 995 996 998 999 1000 1000 1000 1003 1003 1003 1007 1008
1009 1009 1013 1026 1026 1030 1030 1030 1034 1034 1035 1036 1038 1040 1041
1044 1044 1050 1051 1051 1054 1055 1056 1059 1059 1060 1061 1063 1064 1075
1076 1076 1076 1077 1078 1082 1082 1091 1092 1093 1094 1095 1097 1101 1101
1107 1109 1110 1116 1117 1119 1120 1120 1127 1134 1135 1138 1141 1142 1144
1146 1146 1150 1155 1157 1158 1170 1178 1179 1180 1187 1187 1191 1195 1196
1207 1210 1217 1217 1220 1224 1225 1225 1228 1238 1241 1253 1264 1265 1270
1275 1278 1278 1286 1296 1297 1300 1301 1302 1305 1309 1310 1313 1329 1330
1331 1334 1336 1337 1338 1341 1345 1347 1360 1362 1376 1377 1381 1383 1384
1384 1384 1384 1390 1393 1394 1403 1403 1410 1419 1429 1435 1445 1446 1446
1455 1455 1456 1458 1462 1471 1475 1477 1478 1491 1495 1506 1506 1521 1522
1523 1553 1559 1561 1567 1567 1571 1572 1576 1578 1585 1590 1600 1602 1604
1608 1611 1616 1616 1627 1628 1630 1631 1631 1634 1639 1642 1644 1646 1671
1675 1679 1687 1692 1696 1699 1699 1700 1710 1711 1720 1731 1735 1740 1747
1752 1777 1777 1781 1792 1795 1796 1801 1802 1807 1814 1822 1822 1841 1846
1847 1850 1879 1882 1891 1895 1900 1914 1915 1917 1929 1944 1948 1950 1974
1979 1994 2000 2010 2024 2035 2056 2056 2096 2098 2098 2107 2116 2135 2137
2172 2178 2201 2204 2214 2227 2269 2337 2342 2342 2345 2353 2417 2417 2444
2448 2463 2463 2464 2477 2495 2507 2525 2542 2544 2547 2555 2557 2568 2582
2629 2660 2715 2715 2727 2748 2776 2803 2816 2833 2842 2847 2935 2949 2965
2985 2991 2999 3042 3099 3101 3136 3143 3144 3158 3246 3278 3302 3350 3397
3405 3410 3415 3448 3496 3538 3544 3560 3561 3565 3600 3610 3658 3759 3768
3842 3842 3891 3910 3924 3934 3947 3956 3994 4010 4016 4039 4129 4133 4167
4213 4228 4256 4274 4356 4367 4377 4419 4482 4496 4501 4530 4618 4660 4663
4743 4784 4831 4859 4894 5053 5101 5142 5159 5234 5299 5449 5544 5658 5830
5943 6219 6478 6600 6682 6806 6970 6992 7538 7651 7903 7985 8264 9359 9514
9970)))))
  (clt-illustrator sal :variable-labels '("Compensation/1000"))))

(defun cltnorm104 ()
  (clt-illustrator (+ 10 (* 4 (normal-rand 250))) 
   :variable-labels '("N(10,4) random numbers")))

(defun cltexp ()
  (clt-illustrator (exp (uniform-rand 250))
   :variable-labels '("Exponential random numbers")))

(defun cltcauchy ()
  (clt-illustrator (apply #'/ (normal-rand '(250 250)))
   :variable-labels '("Cauchy random numbers")))

(clt-menu)
