;;;; Copyright (C) 1994, 1999  R. Dennis Cook and Sanford Weisberg.  
;;;; A demonstration program to accompany the book \"Introduction to
;;;; Regression Graphics\", New York:  Wiley, Inc., 1994.

(defun 3d-demos ()
  (let* ((menu (send menu-proto :new "Demos:3D"))
         (titles '("View a surface" 
                   "Rotation in 2D"
                   "Detect a small nonlinearity" 
                   "Colinearity hiding a curve"
                   "E[y|x] linear, predictors uncorrelated"
                   "E[y|x] linear, predictors correlated"
                   "E[y|x] linear, predictors nonlinear"
                   "E[y|x] linear, uncorr. pred, x2 not needed"
                   "E[y|x] linear, corr. pred,  x2 not needed"
                   "Interaction"
                   "E[y|x] nonlinear, predictors linear"
                   "E[y|x] nonlinear, predictors nonlinear"
                   "1D structure"
                   "2D structure"
                   "Quit"))
         (actions (list #'sins 
								#'sin2d 
                        #'case5 
								#'case6 
                        #'uncor-pred 
								#'corr-pred
                        #'nonlin-pred
                        #'uncor-nox2 
								#'corr-nox2
                        #'case7 
								#'lin-nonlin 
								#'nonlin-nonlin
						;	   #'lin-nonlin1
								#'1d-structure
                        #'2d-structure
                        #'(lambda () (send menu :remove)))))
  (apply #'send menu :append-items
    (mapcar #'(lambda (title action) 
      (send menu-item-proto :new title :action action)) titles actions))
  (send menu :install)
  menu))


(defun spin1 (data &key variable-labels title)
  (let* ((p (spin-plot data :title title)))
        (send p :plot-controls)
        (send p :install-label-controls variable-labels
                         (send p :variable-label '(0 1 2)))
        p))


(defun make-data (x b e) (list (select x 0) (make-y x b e) (select x 1)))
(defun make-y (x b e) (apply #'+ (select b 0) e (mapcar #'* (rest b) x)))

(defun sins ()
   (let* ((x1 (* 2 pi (select zz 0)))
          (x2 (* 2 pi (select zz 1)))
          (y (+ (sin x1) (* 2 (cos x2)) (* (sin x1) (cos x1))))
          (p (spin1 (list x1 y x2) :variable-labels '("X" "Y" "Z")
                    :title "A 3D Plot")))
         p))


(defun sin2d ()
   (let* ((x1 (* 2 pi (select zz 0)))
          (x2 (* 2 pi (select zz 1)))
          (y (+ (sin x1) (* 2 (cos x2)) (* (sin x1) (cos x1))))
          (p (2d3d x1 y x2)))
         p))

(defun uncor-pred ()
  (let* ((x1 (select xx 0))
         (x2 (select xx 1))
         (ygx (+ 1 (* 2 x1) (* 3 x2)))
         (y (+ e ygx)))
   (r-code :data (list x1 x2 ygx e y)
           :data-names (list "x1" "x2" "E[y|x]" "epsilon" "y")
           :info '("N(0,1)" "N(0,1)" "1 + 2*x1 + 3*x2" "N(0,1)" "E[y|x] + e")
           :description 
"Uncorrelated predictors, linear model is true.  Simulated data, from
3D demos in R. D. Cook and S. Weisberg (1994), \"An Introduction to 
Regression Graphics,\" New York:  Wiley."
           :menu "Uncorr")))

(defun corr-pred ()
  (let* ((x1 (select xx 0))
         (x2 (+ (* .99 x1) (* .141067 (select xx 1))))
         (ygx (+ 1 (* 2 x1) (* 3 x2)))
         (y (+ e ygx)))
   (r-code :data (list x1 x2 ygx e y)
           :data-names (list "x1" "x2" "E[y|x]" "epsilon" "y")
           :info '("N(0,1)" ".99*x1 + 141*N(0,1)" "1 + 2*x1 + 3*x2" "N(0,1)" 
                   "E[y|x] + e")
           :description 
"Correlated predictors, linear model is true.  Simulated data, from
3D demos in R. D. Cook and S. Weisberg (1994), \"An Introduction to 
Regression Graphics,\" New York:  Wiley."
           :menu "Corr")))

(defun nonlin-pred ()
  (let* ((x1 (select xx 0))
         (x2 (+ (^ (select xx 0) 2) (* .5 (select xx 1))))
         (ygx (+ 1 (* 2 x1) (* 3 x2)))
         (y (+ e ygx)))
   (r-code :data (list x1 x2 ygx e y )
           :data-names (list "x1" "x2" "E[y|x]" "epsilon" "y" )
           :info '("N(0,1)" "x1^2 + .5*N(0,1)" "1 + 2*x1 + 3*x2" "N(0,1)" 
                   "E[y|x] + e")
           :description 
"The mean function E[y|x] is linear, but predictors are not all 
normally distributed or linearly related.  Simulated data, from
3D demos in R. D. Cook and S. Weisberg (1994), \"An Introduction to
Regression Graphics,\" New York:  Wiley."
           :menu "NonLin")))

(defun uncor-nox2 ()
  (let* ((x1 (select xx 0))
         (x2 (select xx 1))
         (ygx (+ 1 (* 2 x1) (* 0 x2)))
         (y (+ e ygx)))
   (r-code :data (list x1 x2 ygx e y)
           :data-names (list "x1" "x2" "E[y|x]" "epsilon" "y")
           :info '("N(0,1)" "N(0,1)" "1 + 2*x1" "N(0,1)" 
                   "E[y|x]+e")
           :description 
"The mean function E[y|x] is linear, but x2 is not needed.  Simulated data, 
from 3D demos in R. D. Cook and S. Weisberg (1994), \"An Introduction to
Regression Graphics,\" New York:  Wiley."
           :menu "NoX2")))

(defun corr-nox2 ()
  (let* ((x1 (select xx 0))
         (x2 (+ (* .95 x1) (* .31225 (select xx 1))))
         (ygx (+ 1 (* 2 x1) (* 0 x2)))
         (y (+ e ygx)))
   (r-code :data (list x1 x2 ygx e y)
           :data-names (list "x1" "x2" "E[y|x]" "epsilon" "y")
           :info '("N(0,1)" ".95*x1 + .31*N(0,1)" "1 + 2*x1" "N(0,1)" 
                   "E[y|x]+e")
           :description 
"The mean function E[y|x] is linear. Although x2 is not needed, it is
correlated with x1.  Simulated data, from 3D demos in R. D. Cook and 
S. Weisberg (1994), \"An Introduction to Regression Graphics,\" New 
York:  Wiley."
           :menu "NoX2-Corr")))

(defun lin-nonlin ()
  (let* ((x1 (select xx 0))
         (x2 (select xx 1))
;        (ygx (^ (+ 2 (* 2 x1)) 2))  original version
         (ygx (* 3 (exp (/ (+ 2 (* 2 x1)) 2.5))))
         (y (+ e ygx)))
   (r-code :data (list x1 x2 ygx e y )
           :data-names (list "x1" "x2" "E[y|x]" "epsilon" "y" )
           :info '("N(0,1)" "N(0,1)" 
                   "3*exp((2+2*x1)/2.5)"   ;"(2 + 2*x1)^2" 
                   "N(0,1)" 
                   "E[y|x]+e")
           :description 
"The mean function E[y|x] is NOT linear. The predictors are uncorrelated.
Simulated data, from 3D demos in R. D. Cook and S. Weisberg (1994), 
\"An Introduction to Regression Graphics,\" New York:  Wiley."
           :menu "Lin/NonLin")))

(defun lin-nonlin1 ()
  (let* ((x1 (select xx 0))
         (x2 (select xx 1))
         (ygx (^ (+ 2 (* 2 x1) (- x2)) 2))
         (y (+ (* 4 e) ygx)))
   (r-code :data (list x1 x2 ygx e y )
           :data-names (list "x1" "x2" "E[y|x]" "epsilon" "y" )
           :info '("N(0,1)" "N(0,1)" "(2 + 2*x1 - x2)^2" "N(0,4^2)" 
                   "E[y|x]+e")
           :description 
"The mean function E[y|x] is NOT linear. The predictors are uncorrelated.
Simulated data, from 3D demos in R. D. Cook and S. Weisberg (199x), 
\"An Introduction to Regression Graphics,\" New York:  Wiley."
           :name "D1")))

(defun 1d-structure ()
  (let* ((x1 (select xx 0))
         (x2 (select xx 1))
         (btx (- (* 2 x1) x2))
         (ygx (+ 1 (* 2 btx) (* 3 (^ btx 2))))
         (y (+ (* 10 e) ygx)))
   (r-code :data (list x1 x2 ygx e y )
           :data-names (list "x1" "x2" "E[y|x]" "epsilon" "y" )
           :info '("N(0,1)" "N(0,1)" "1+2(2x1-x2)+3(2x1-x2)^2" "N(0,10^2)" 
                   "E[y|x]+e")
           :description 
"Simulated data.  From 3D demos in R. D. Cook and S. 
Weisberg (1994), \"An Introduction to Regression Graphics,\" 
New York:  Wiley."
           :name "D1")))

(defun nonlin-nonlin ()
  (let* ((x1 (select xx 1))
         (x2 (+ (^ x1 2) (select xx 0)))
;        (ygx (^ (+ 2 (* 2 x1)) 2))
         (ygx (* 3 (exp (/ (+ 2 (* 2 x1)) 2.5))))
         (y (+ e ygx)))
   (r-code :data (list x1 x2 ygx e y )
           :data-names (list "x1" "x2" "E[y|x]" "epsilon" "y" )
           :info '("N(0,1)" "x1^2 + N(0,1)" 
                   "3*exp((2+2*x1)/2.5)"   ;"(2 + 2*x1)^2" 
                   "N(0,1)" 
                   "E[y|x]+e")
           :description 
"The mean function E[y|x] is NOT linear. The predictors are not linearly
related.  Simulated data, from 3D demos in R. D. Cook and S. Weisberg (1994), 
\"An Introduction to Regression Graphics,\" New York:  Wiley."
           :menu "NonLin/Nonlin")))

(defun case5 ()
  (let* ((x1 (repeat (iseq 1 10) 10))
         (x2 (repeat (iseq 1 10) (repeat 10 10)))
         (u (+ (* (sin 2) x1) (* (cos 2) x2)))
         (y (+ u (/ (exp (- u)) (+ 1 (exp (- u))))))
         (p (spin1 (list x1 y x2) :variable-labels '("X1" "Y" "X2")
                       :title "Needs detrending")))))

(defun case6 (&key (dataset nil))
"This function creates a demo using the stored values of x and e.  The
response y = .5x1 -x2 +1.5(x1-x2)^2+4e, so the error has SD=4.  The plot is of
\{Z1,Y,Z2\}, where Z is obtained from x by multiplying by a matrix that will
give them correlation of 0.99."
  (let* ((rt (first (chol-decomp (bind-columns '(1 .99) '(.99 1)))))
         (z (row-list (matmult rt (apply #'bind-rows xx))))
         (y (+ (* .5 (first xx)) (* -1 (second xx)) 
               (* 1.5 (^ (- (first xx) (second xx)) 2)) (* 4 e)))
         (d (when dataset (rcode :data (append z (list y))
                            :data-names '("X1" "X2" "Y")
                            :name "Collin"
                            :description "Simulated Data")))
         (p (spin1 (list (first z) y (second z)) 
              :variable-labels '("X1" "Y" "X2")
               :title "Collinearity is Hiding a curve" 
               )))))

(defun case7 ()
  (let* ((x (* 2 xx))
         (p (spin1 (list (select x 0)
             (+ (select x 0) (select x 1) (* (select x 0) (select x 1)) e)
             (select x 1))
             :variable-labels '("X1" "Y" "X2")
             :title "Interaction should be apparent")))))

(defun 2d-structure ()
  (let* ((x (* 2 xx))
         (eygx (+ (select x 0) (select x 1) (* (select x 0) (select x 1)) ))
         (y (+ eygx e)))
  (rcode :data (list (select x 0) (select x 1) y eygx e)
             :data-names '("x1" "x2" "y" "E[y|x]" "e")
             :info '("N(0,1)" "N(0,1)" "E[y|x]+e" "2*x1+2*x2+4*x1*x2" "N(0,1)")
             :name "D2"
             :description
"A simulated example requiring 2D structure")))

(defun 2d3d (x1 y x2) 
"Used by the function sin2d."
  (flet ((scale (x)
                (let* ((r (range x)))
                      (* (- x (* .5 (apply #'+ r))) 
                         (/ 2 (apply #'- (reverse r)))))))
    (let* ((x1 (scale  x1))
           (y (scale y))
           (x2 (scale x2))
           (p (plot-points x1 y :variable-labels '("h(theta)" "Y")
                           :title "Dynamic 2D plot" :show (showplot)))
           (top (+ 8 (* 2 (+ (send p :text-ascent) (send p :text-descent)))))
           slider)
          (apply #'send p :margin (+ (send p :margin) (list 0 top  0  0)))
          (send p :add-slot 'overlay-loc '(10 2))
          (apply #'send p :size (+ (send p :size) (list 0 top)))
          (send p :resize)
          (def slider (send slider-control-proto :new
                               (rseq 0 (* 2 pi) 37)
                               :location (send p :locate-next-control :height 2)
                               :display (* (iseq 37) 10)
                               :title "Theta"
                               :length (send p :slider-width)))
          (defmeth slider :do-action (theta)
                   (send p :draw-next-frame '(0)
                         (list (+ (* (cos theta) x1)
                                  (* (sin theta) x2)))))
          (send p :add-overlay slider)
          (send p :point-selected (iseq (send p :num-points)) t)
          (send p :range 0 -1.5 1.5)
          (send p :show-window)
          p)))

;;;
;;;  Data was generated randomly, and then stored
;;;

(DEF XX (QUOTE ((0.408406 0.463656 0.904599 -1.38257 1.25267 0.558381 -0.814229
-1.1278 0.70464 -1.2796 0.140687 0.251646 0.0291833 -0.344757 -1.93948
-1.00045 -1.3278 0.512824 0.327152 -0.851935 0.622949 -1.04232 -0.526875
-0.817473 -0.870402 2.05864 -0.160695 0.928031 -0.692586 -0.282536 0.199642
0.52652 -0.157993 -0.634574 -0.50803 -1.24954 -1.38543 -2.45606 -0.519377
-0.720627 -0.951436 0.493766 -0.98149 1.21623 0.358366 0.15909 -0.800093
1.84132 0.569207 -0.203394 -0.202613 0.257011 0.798855 -0.515437 1.83367
0.175605 -0.964146 -1.41545 1.15412 0.244984 0.685545 0.2518 1.52664 1.45174
0.695117 0.327983 0.705825 -0.185056 -0.654103 1.31298 0.170931 0.58719
0.101512 -1.80721 -1.54562 -1.59503 1.36447 0.584018 1.01552 0.756013 0.843221
-0.490341 -0.302421 -0.793353 0.82801 -1.41015 -0.348409 -0.255372 1.68179
1.25352 0.796617 1.31766 0.170576 -0.158776 -0.779074 0.492604 0.315616
-0.517257 1.5965 -0.752115) (0.839849 0.721682 1.50226 0.0389832 -0.366002
-1.52769 0.472236 -2.71291 0.653931 -1.91164 0.212134 0.321642 1.68644
0.998239 -0.397621 -0.636527 -0.127016 -0.0717705 0.383598 -1.0252 0.687385
0.380374 0.313442 0.955189 1.47978 -1.32328 0.130161 0.977379 2.16244 -1.04838
-0.755727 -0.710983 0.798051 -1.41223 0.974521 -0.976681 -2.16693 -0.0206029
-1.60182 -2.39414 0.472581 -0.232011 0.517098 -0.685697 -0.352487 0.700035
2.3091 0.316756 -0.777417 -0.207738 0.959395 1.09295 -1.51179 0.105229
-1.33423 0.226059 -0.480993 0.02681 -0.881284 -0.152642 -1.98237 1.03629
-1.30691 0.00033328 0.859041 0.172521 -0.130896 2.52911 1.32 0.676581 -1.20577
-1.16174 3.26922 -0.250669 0.0752248 -0.840335 0.144739 -0.652078 -0.572697
0.796997 -0.540887 0.616131 -0.00315169 0.0224316 0.842157 0.695943 -1.01269
0.00897507 -0.238497 0.599486 -1.71469 0.491624 0.300711 0.843289 0.189044
0.516132 0.293386 2.31679 2.38415 1.47917))))
(DEF E (QUOTE (-1.09386 -0.103222 0.426581 -0.648306 1.22626 -0.035458
-0.583101 -1.6982 0.391416 0.738056 -1.4804 -0.0783676 0.675759 -1.92781
0.209448 -2.41878 -0.29025 0.836687 -0.0110639 0.112623 -0.700727 0.935346
-0.760566 -0.0857465 0.567596 0.542026 1.82198 0.164534 1.97605 0.136114
-0.774758 0.141412 -0.768173 1.86695 -0.431941 -1.05779 -0.215175 0.00469343
-0.253323 1.60639 -0.8453 0.670222 0.68416 1.41866 -0.702795 0.31476 -0.435515
-1.41174 -0.928143 -1.44194 1.93177 -1.2739 0.619966 -0.67047 -0.102337
0.793336 -0.0832021 0.442864 -0.680893 0.508127 0.489781 -0.315737 0.0611294
0.813888 1.34519 -0.590844 -0.26773 1.40588 1.87036 0.0167546 0.592218
-0.788162 -0.209649 -1.59043 1.23822 -1.56999 0.152774 0.392133 0.951851
0.67872 0.552667 1.62891 1.10321 -1.77708 -0.141641 1.12452 0.86704 0.491455
0.195366 1.00043 -0.00870563 0.537871 0.178713 1.51749 1.21714 -0.268516
0.92842 1.10195 0.516473 1.12529)))
(DEF ZZ (QUOTE ((0.0148848 0.221329 0.535946 0.321338 0.480197 0.686992 0.19086
0.186461 0.0152726 0.42406 0.889738 0.397715 0.662238 0.712665 0.645739
0.742903 0.794431 0.647353 0.491336 0.109793 0.421565 0.314234 0.960361
0.300476 0.923332 0.406292 0.890174 0.0706234 0.902761 0.261093 0.693628
0.244435 0.32772 0.10833 0.61374 0.202292 0.134642 0.906155 0.794096 0.653379
0.901816 0.211311 0.499863 0.903922 0.582755 0.999055 0.950217 0.806236
0.659486 0.255035 0.890726 0.336477 0.603944 0.524844 0.34888 0.0966302
0.683098 0.702128 0.313534 0.849017 0.192709 0.100343 0.703073 0.363317
0.042781 0.533222 0.845307 0.812347 0.0268397 0.438837 0.00837815 0.496427
0.715717 0.343742 0.736709 0.694845 0.647411 0.523008 0.243399 0.0336365
0.331528 0.60463 0.989786 0.398092 0.22129 0.304688 0.165793 0.981408 0.901664
0.505573 0.960946 0.429084 0.286563 0.254254 0.982565 0.717547 0.395447
0.955035 0.649624 0.992779) (0.319455 0.174157 0.650347 0.483831 0.0113715
0.417791 0.668584 0.689401 0.0547476 0.724808 0.163537 0.68602 0.971853
0.659301 0.769773 0.513914 0.69324 0.652398 0.485143 0.119426 0.0300824
0.681869 0.234607 0.816559 0.430026 0.975335 0.957061 0.0710694 0.130539
0.458172 0.316034 0.187287 0.557156 0.437299 0.805775 0.830891 0.0678611
0.527073 0.75543 0.571168 0.0143324 0.637835 0.551739 0.798369 0.500098
0.883793 0.179663 0.235704 0.611082 0.942943 0.446494 0.373888 0.404813
0.543221 0.415869 0.691065 0.802721 0.390481 0.905386 0.864131 0.892695
0.302622 0.506688 0.725723 0.628426 0.281613 0.359679 0.0601932 0.351834
0.223613 0.738392 0.94381 0.369128 0.549114 0.833132 0.833007 0.0796793
0.476433 0.246492 0.326445 0.107284 0.451253 0.19482 0.886812 0.266252 0.75545
0.22764 0.456427 0.943002 0.897123 0.206337 0.394507 0.62342 0.863323 0.42069
0.959845 0.0680624 0.516136 0.41207 0.22587))))


;;;
;;; execute the code
;;;

(3d-demos)
