(defun clt-illustrator (y &rest args
  &key (sample-sizes '(2 3 4 5 6 7 8 16 32 64)) (reps 500))
"Functions args:  (y &key (sample-sizes '(2 3 4 8 16 32)) (reps 500))
Draws a histogram of y, and adds a normal density with the same mean and sd.
Opens a second histogram window with data equal to reps sample means of size
sample-sizes, with the corresponding normal density added. A slide bar allows
changing the sample size."
 (flet ((get-means (n) (mapcar #'(lambda (r) (mean
                  (select y (floor (* (length y) (uniform-rand n))))))
                  (iseq reps))))
  (let* ((h1 (apply #'histogram y :title "Raw data" args))
         (ybar (mean y))
         (ysd (standard-deviation y))
         (j 0)
         (n (select sample-sizes j))
         (h2 (histogram (get-means n) :title "Histogram of means"
               :variable-labels (list (format nil "Means of ~a" n)) 
               :show nil))
         (fixed-range t)
         (rangex (send h2 :range 0)))
   (send h1 :plot-controls)
   (send h1 :add-function
       #'(lambda (z) (/ (normal-dens (/ (- z ybar) ysd)) ysd))
       (- ybar (* 3.5 ysd)) (+ ybar (* 3.5 ysd)))
   (defmeth h2 :install-means-control ()
     (let ((c (first (send self :add-overlay
                 (send slider-control-proto :new sample-sizes
                    :title "Sample size" :popup t
                    :location (send self :locate-next-control :height 2)
                    :length (send self :slider-width))))))
      (defmeth c :menu (&optional (title "Scaling Option")) 
        (let* ((graph (send self :graph)))
          (when (null (slot-value 'menu)) 
            (let* ((menu (send menu-proto :new title)))
                  (apply #'send menu :append-items (send self :get-menu-items))
                  (send self :slot-value 'menu menu)))
              (slot-value 'menu)))
      (defmeth c :shift-click ()
        (call-method smoother-proto :shift-click))
      (defmeth c :get-menu-items ()
        (let* ((adjust (send menu-item-proto :new "Adjust Scale"
              :action #'(lambda () (setf fixed-range (null fixed-range))))))
          (defmeth adjust :update ()
             (send adjust :mark (null fixed-range)))
          (list adjust)))
      (defmeth c :do-action (a)
        (send h2 :start-buffering)
        (send h2 :clear-lines)
        (send h2 :point-coordinate 0 (iseq reps) (get-means a))
        (send h2 :normal-density)
        (send h2 :variable-label 0 (format nil "Means of ~a" a))
        (send h2 :adjust-to-data)
        (if fixed-range (apply #'send h2 :range 0 rangex))
        (send h2 :buffer-to-screen))))
   (defmeth h2 :plot-controls ()
     (call-method graph-proto :plot-controls)
     (send self :install-bin-control)
     (send self :install-kernel-control)
     (send self :install-means-control)
     t)
   (send h2 :plot-controls)
   (defmeth h2 :normal-density ()
     (let* ((y (send h2 :point-coordinate 0 (iseq reps)))
            (ybar (mean y))
            (ysd (standard-deviation y)))
      (send self :add-function
        #'(lambda (z) (/ (normal-dens (/ (- z ybar) ysd)) ysd))
        (- ybar (* 3.5 ysd)) (+ ybar (* 3.5 ysd)))))
   (send h2 :normal-density)
   (apply #'send h2 :location (+ (list 0 45)
     (send h1 :location) (list 0 (second (send h1 :size)))))
   (setf g1 h1)
   (setf g2 h2)
   (send h2 :show-window))))


(defun clt-menu ()
   (let* ((menu (send menu-proto :new "CLT"))
          (items '( "Haystack volume"
                    "N(10,4) random numbers"
                    "Highly skewed exponential data"
                    "Cauchy data"
                    "Quit"))
          (actions (list #'clthay #'cltnorm104 #'cltexp #'cltcauchy
                   #'(lambda () (send menu :remove)))))
   (apply #'send menu :append-items
          (mapcar #'(lambda (i a) (send menu-item-proto :new i :action a))
            items actions))
   (send menu :install)
   menu))

(defun clthay ()
 (let ((vol (* .001
(QUOTE (2853 2702 3099 1306 2294 2725 3309 2790 2756 5237.92 3149.82 5498.46
3397.83 3007.56 4574.29 6228.59 2318.64 3176.71 2352.31 2174.44 2694.72
3333.53 4328.92 2115.04 2489.08 2296.65 3117.21 4088.36 4180.88 2318.19 1946.9
2479.89 3174.8 2151.54 3475.68 4393.08 2819.5 3703.49 2742.81 3002.4 1854.19
1982.07 2470.86 1203.15 2843.84 2636.25 1998.39 2005.03 2568.76 2161.18 2112.2
3009.33 1992.24 2746.98 2238.27 1747.47 2863.91 3593.47 2435.48 2430.18
2590.07 3577.68 3299.24 1986.14 3109.04 2821.56 2932.24 3304.63 2565.46
4509.93 4804.01 4241.8 4516.1 5011.62 2110.73 2775.7 3927.9 4212.77 3562.64
2853.96 3294.38 1689.54 2228.84 2362.61 3088.28 3820.79 3126.64 3624.75
3023.97 6045.42 3100.11 3378.07 3040.29 2252.16 3552.61 2635.9 3201.41 2590.21
3743.55 3858.03 3829.44 2556.44 3119.07 2122.38 2921.92 2936.35 2427.66
2069.38 1899.54 4289.28 2407.39 3097.99 3893.67 2238.66 2314.79 2667.07
2582.07 3426.5 2307.34 3960.41)))))
  (clt-illustrator vol :variable-labels '("Vol/1000"))))

(defun cltnorm104 ()
  (clt-illustrator (+ 10 (* 4 (normal-rand 250))) 
   :variable-labels '("N(10,4) random numbers")))

(defun cltexp ()
  (clt-illustrator (exp (uniform-rand 250))
   :variable-labels '("Exponential random numbers")))

(defun cltcauchy ()
  (clt-illustrator (apply #'/ (normal-rand '(250 250)))
   :variable-labels '("Cauchy random numbers")))

(clt-menu)
