;;;; Copyright (C) 1994 R. Dennis Cook and Sanford Weisberg.  The R-code is 
;;;; XLISP-STAT computer code to accompany the book \"Introduction to
;;;; Regression Graphics\", to be published by John Wiley, Inc.
;;;; Until publication of the book in 1994,
;;;; the program may be used by students at the University of
;;;; Minnesota, or by others with the written permission of the
;;;; copyright holders. 

(defun conf-region-demo (&key (df 100) (level .95))
"Function args: ()
This is a demo program that draws a bivariate confidence region for
\etahat_1, \etahat_2 in a 2 variable regression with the predictors centered
and scaled to have mean 0 and sd 1, and the response scaled to have \sigmahat
= 1.  In this case, X'X is a correlation matrix, with the off diagonal element
the correlation, rho.  The scaling is equivalent to scaling the axes of the
plot, and is therefore irrelevant.  As all intervals are ellipses, the number
of degrees of freedom are also irrelevant."
 (let* ((fmult (sqrt (* 2 (f-quant level 2 df))))
        (tmult (- (t-quant (* .5 (- 1 level)) df)))
        (bmult (- (t-quant (* .25 (- 1 level)) df)))
        (blims (list  (- bmult) bmult))
        (p (plot-lines (ellipse (matrix '(2 2) '(1 0 0 1)) fmult '(0 0))
                       :title "Confidence Regions"
                       :variable-labels '("Eta1" "Eta2")
                       :color 'blue :show (showplot)
			  ))
        (lims (list (- tmult) tmult))
;        (button (send p :install-bonferroni-button))
        (top (+ 8 (* 2 (+ (send p :text-ascent) (send p :text-descent)))))
        slider)
    (apply #'send p :margin (+ (send p :margin) (list 0 top  0  0)))
	 (send p :add-slot 'overlay-loc '(10 2))
	 (send p :resize)
    (def slider (send slider-control-proto :new
                      (combine  -.95 -.9 (rseq -.8 .8 9) .9 .95)
                      :location (send p :locate-next-control :height 2)
                      :graph p :length (send p :slider-width)
                      :title "r12"))
    (send slider :index 6)
    (setf slide slider)
    (defmeth slider :do-action (rho) 
             (send p :start-buffering)
             (send p :clear-lines)
             (setf blims (list (- (/ bmult (sqrt (- 1 (^ rho 2)))))
                          (/ bmult (sqrt (- 1 (^ rho 2))))))
        (let ((lims (list (- (/ tmult (sqrt (- 1 (^ rho 2)))))
                          (/ tmult (sqrt (- 1 (^ rho 2)))))))
             (send p :add-lines
                    (ellipse (matrix '(2 2) (list 1 rho rho 1)) fmult '(0 0))
                    :color 'blue)
             (send p :add-lines (list lims '(0 0)) :color 'red)
             (send p :add-lines (list '(0 0) lims) :color 'red)
;            (when (send button :slot-value 'selected)
;                  (send button :do-action))
             (send self :draw-indicator (send self :index))
#+msdos (send p :redraw)
             (send p :buffer-to-screen)))
;   (defmeth button :do-action ()
;      (setf (slot-value 'selected) t)
;      (send self :bitmap t)
;      (send p :add-lines (select blims '(0 1 1 0 0))
;               (select blims '(0 0 1 1 0)) :color 'magenta))
    (send p :range 0 -10 10)
    (send p :range 1 -10 10)
    (send p :add-points (list 0) (list 0) :color 'blue)
    (send p :add-lines (list lims (list 0 0)) :color 'red) 
    (send p :add-lines (list (list 0 0) lims) :color 'red) 
    (send p :show-window)
       p))

(defmeth graph-proto :install-bonferroni-button ()
  (call-method graph-proto :plot-controls)
  (let ((button (send checkbox-control-proto :new
    :title "Bonferroni Bounds" :location (send self :locate-next-control))))
       (send self :add-overlay button)
       button))

(defun ellipse (M r c &key (npoints 100))
"Function args: (M r c &key (npoints 100))
Returns the cartesian coordinates of points z on the ellipse
                  (z-c)'M(z-c) = r^2,
where z = z(theta) and theta varies from 0 to (* 2 pi) radians in npoints
steps."
; lam gives inverse of square root of the eigenvalues of M
; gamma is the matrix of eigenvectors or left singular vectors
   (let* ((lam (/ (sqrt (eigenvalues m))))
          (gamma (apply #'bind-columns (eigenvectors m))))
    (labels ((circle (theta r) (* r (list (cos theta) (sin theta))))
             (ellip (theta r lambda) (* lambda (circle theta r)))
             (point (theta) (+ c
                   (coerce (matmult gamma (ellip theta r lam)) 'list))))
          (transpose (mapcar #'point (rseq 0 (* 2 pi) npoints))))))

(defun source () (format t "~%
In this demo, we have a regression problem with two predictors x1 and 
x2 scaled to have mean zero and sum of squared deviations equal to 1.  
We fit the model with mean function E(y|x)= eta_0+eta_1x1 + eta_2x2.
The initial plot is the joint 95\% confidence region for etahat1, 
etahat2; the crossed lines give the marginal intervals, assuming that 
the point estimates are at (0, 0), and the estimated sigmahat = 1.  
These intervals make no correction for multiple testing.  The slide bar 
controls the correlation r12 between X1 and X2, and hence the correlation 
-r12 between etahat1 and etahat2.  As the slide bar is moved, the 
confidence regions are redrawn.~%"))
(source)
(conf-region-demo)
